@@ -12,7 +12,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-# Shell script tool to run puppet inside of the given docker container image.
+# Shell script tool to run puppet inside of the given container image.
 # Uses the config file at /var/lib/docker-puppet/docker-puppet.json as a source for a JSON
 # array of [config_volume, puppet_tags, manifest, config_image, [volumes]] settings
 # that can be used to generate config files or run ad-hoc puppet modules
@@ -28,10 +28,10 @@ import tempfile
 import time
 import multiprocessing
 
-from paunch import runner as containers_runner
-
 logger = None
-RUNNER = containers_runner.DockerRunner('docker-puppet')
+sh_script = '/var/lib/docker-puppet/docker-puppet.sh'
+container_cli = os.environ.get('CONTAINER_CLI', 'docker')
+cli_cmd = '/usr/bin/' + container_cli
 
 
 def get_logger():
@@ -52,18 +52,41 @@ def get_logger():
     return logger
 
 
+log = get_logger()
+log.info('Running docker-puppet')
+
+config_volume_prefix = os.environ.get('CONFIG_VOLUME_PREFIX', '/var/lib/config-data')
+log.debug('CONFIG_VOLUME_PREFIX: %s' % config_volume_prefix)
+if not os.path.exists(config_volume_prefix):
+    os.makedirs(config_volume_prefix)
+
+if container_cli == 'docker':
+    cli_dcmd = ['--volume', '/usr/share/openstack-puppet/modules/:/usr/share/openstack-puppet/modules/:ro,z']
+    env = {}
+elif container_cli == 'podman':
+    # podman doesn't allow relabeling content in /usr and
+    # doesn't support named volumes
+    cli_dcmd = ['--security-opt', 'label=disable',
+                '--volume', '/usr/share/openstack-puppet/modules/:/usr/share/openstack-puppet/modules/:ro']
+    # podman need to find dependent binaries that are in environment
+    env = {'PATH': os.environ['PATH']}
+else:
+    log.error('Invalid container_cli: %s' % container_cli)
+    sys.exit(1)
+
+
 # this is to match what we do in deployed-server
 def short_hostname():
     subproc = subprocess.Popen(['hostname', '-s'],
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
     cmd_stdout, cmd_stderr = subproc.communicate()
-    return cmd_stdout.rstrip()
+    return cmd_stdout.decode('utf-8').rstrip()
 
 
 def pull_image(name):
 
-    subproc = subprocess.Popen(['/usr/bin/docker', 'inspect', name],
+    subproc = subprocess.Popen([cli_cmd, 'inspect', name],
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.PIPE)
     cmd_stdout, cmd_stderr = subproc.communicate()
@@ -77,7 +100,7 @@ def pull_image(name):
     log.info('Pulling image: %s' % name)
     while retval != 0:
         count += 1
-        subproc = subprocess.Popen(['/usr/bin/docker', 'pull', name],
+        subproc = subprocess.Popen([cli_cmd, 'pull', name],
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)
 
@@ -85,7 +108,7 @@ def pull_image(name):
         retval = subproc.returncode
         if retval != 0:
             time.sleep(3)
-            log.warning('docker pull failed: %s' % cmd_stderr)
+            log.warning('%s pull failed: %s' % (container_cli, cmd_stderr))
             log.warning('retrying pulling image: %s' % name)
         if count >= 5:
             log.error('Failed to pull image: %s' % name)
@@ -122,7 +145,7 @@ def get_config_hash(config_volume):
 def rm_container(name):
     if os.environ.get('SHOW_DIFF', None):
         log.info('Diffing container: %s' % name)
-        subproc = subprocess.Popen(['/usr/bin/docker', 'diff', name],
+        subproc = subprocess.Popen([cli_cmd, 'diff', name],
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)
         cmd_stdout, cmd_stderr = subproc.communicate()
@@ -132,7 +155,7 @@ def rm_container(name):
             log.debug(cmd_stderr)
 
     log.info('Removing container: %s' % name)
-    subproc = subprocess.Popen(['/usr/bin/docker', 'rm', name],
+    subproc = subprocess.Popen([cli_cmd, 'rm', name],
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
     cmd_stdout, cmd_stderr = subproc.communicate()
@@ -145,8 +168,6 @@ def rm_container(name):
 
 process_count = int(os.environ.get('PROCESS_COUNT',
                                    multiprocessing.cpu_count()))
-log = get_logger()
-log.info('Running docker-puppet')
 config_file = os.environ.get('CONFIG', '/var/lib/docker-puppet/docker-puppet.json')
 # If specified, only this config_volume will be used
 config_volume_only = os.environ.get('CONFIG_VOLUME', None)
@@ -176,6 +197,7 @@ for service in (json_data or []):
             service.get('step_config'),
             service.get('config_image'),
             service.get('volumes', []),
+            service.get('privileged', False),
         ]
 
     config_volume = service[0] or ''
@@ -183,6 +205,7 @@ for service in (json_data or []):
     manifest = service[2] or ''
     config_image = service[3] or ''
     volumes = service[4] if len(service) > 4 else []
+    privileged = service[5] if len(service) > 5 else False
 
     if not manifest or not config_image:
         continue
@@ -192,6 +215,7 @@ for service in (json_data or []):
     log.debug('manifest %s' % manifest)
     log.debug('config_image %s' % config_image)
     log.debug('volumes %s' % volumes)
+    log.debug('privileged %s' % privileged)
     # We key off of config volume for all configs.
     if config_volume in configs:
         # Append puppet tags and manifest.
@@ -218,13 +242,12 @@ for service in (json_data or []):
 
 log.info('Service compilation completed.')
 
-sh_script = '/var/lib/docker-puppet/docker-puppet.sh'
 with open(sh_script, 'w') as script_file:
-    os.chmod(script_file.name, 0755)
+    os.chmod(script_file.name, 0o755)
     script_file.write("""#!/bin/bash
     set -ex
     mkdir -p /etc/puppet
-    cp -a /tmp/puppet-etc/* /etc/puppet
+    cp -dR /tmp/puppet-etc/* /etc/puppet
     rm -Rf /etc/puppet/ssl # not in use and causes permission errors
     echo "{\\"step\\": $STEP}" > /etc/puppet/hieradata/docker_puppet.json
     TAGS=""
@@ -232,13 +255,6 @@ with open(sh_script, 'w') as script_file:
         TAGS="--tags \"$PUPPET_TAGS\""
     fi
 
-    CHECK_MODE=""
-    if [ -d "/tmp/puppet-check-mode" ]; then
-        mkdir -p /etc/puppet/check-mode
-        cp -a /tmp/puppet-check-mode/* /etc/puppet/check-mode
-        CHECK_MODE="--hiera_config /etc/puppet/check-mode/hiera.yaml"
-    fi
-
     # Create a reference timestamp to easily find all files touched by
     # puppet. The sync ensures we get all the files we want due to
     # different timestamp.
@@ -246,12 +262,21 @@ with open(sh_script, 'w') as script_file:
     touch $origin_of_time
     sync
 
+    export NET_HOST="${NET_HOST:-false}"
     set +e
+    if [ "$NET_HOST" == "false" ]; then
+        export FACTER_hostname=$HOSTNAME
+    fi
     # $::deployment_type in puppet-tripleo
     export FACTER_deployment_type=containers
-    export FACTER_uuid=$(cat /sys/class/dmi/id/product_uuid | tr '[:upper:]' '[:lower:]')
-    FACTER_hostname=$HOSTNAME /usr/bin/puppet apply --summarize \
-    --detailed-exitcodes --color=false --logdest syslog --logdest console --modulepath=/etc/puppet/modules:/usr/share/openstack-puppet/modules $TAGS $CHECK_MODE /etc/config.pp
+    /usr/bin/puppet apply --summarize \
+            --detailed-exitcodes \
+            --color=false \
+            --logdest syslog \
+            --logdest console \
+            --modulepath=/etc/puppet/modules:/usr/share/openstack-puppet/modules \
+            $TAGS \
+            /etc/config.pp
     rc=$?
     set -e
     if [ $rc -ne 2 -a $rc -ne 0 ]; then
@@ -267,66 +292,29 @@ with open(sh_script, 'w') as script_file:
                 rsync_srcs+=" $d"
             fi
         done
-        # On stack update, if a password was changed in a config file,
-        # some services (e.g. mysql) must change their internal state
-        # (e.g. password in mysql DB) when paunch restarts them; and
-        # they need the old password to achieve that.
-        # For those services, we update the config hash to notify
-        # paunch that a restart is needed, but we do not update the
-        # password file in docker-puppet if the file already existed
-        # before and let the service regenerate it instead.
-        password_files="/root/.my.cnf"
-
-        exclude_files=""
-        for p in $password_files; do
-            if [ -f "$p" -a -f "/var/lib/config-data/${NAME}$p" ]; then
-                exclude_files+=" --exclude=$p"
-            fi
-        done
-
-        # Exclude read-only mounted directories/files which we do not want
-        # to copy or delete.
-        ro_files="/etc/puppetlabs/ /opt/puppetlabs/"
-        for ro in $ro_files; do
-            if [ -e "$ro" ]; then
-                exclude_files+=" --exclude=$ro"
-            fi
-        done
-
-        rsync -a -R --delay-updates --delete-after $exclude_files $rsync_srcs /var/lib/config-data/${NAME}
+        rsync -a -R --delay-updates --delete-after $rsync_srcs /var/lib/config-data/${NAME}
 
 
         # Also make a copy of files modified during puppet run
         # This is useful for debugging
         echo "Gathering files modified after $(stat -c '%y' $origin_of_time)"
         mkdir -p /var/lib/config-data/puppet-generated/${NAME}
-        rsync -a -R -0 --delay-updates --delete-after $exclude_files \
+        rsync -a -R -0 --delay-updates --delete-after \
                       --files-from=<(find $rsync_srcs -newer $origin_of_time -not -path '/etc/puppet*' -print0) \
                       / /var/lib/config-data/puppet-generated/${NAME}
 
         # Write a checksum of the config-data dir, this is used as a
         # salt to trigger container restart when the config changes
-        # note: while being excluded from the output, password files
-        # are still included in checksum computation
-        additional_checksum_files=""
-        excluded_original_passwords=""
-        for p in $password_files; do
-            if [ -f "$p" ]; then
-                additional_checksum_files+=" $p"
-                excluded_original_passwords+=" --exclude=/var/lib/config-data/*${p}"
-            fi
-        done
-
         # We need to exclude the swift rings and their backup as those change over time and
         # containers do not need to restart if they change
-        EXCLUDE=--exclude='*/etc/swift/backups/*'\ --exclude='*/etc/swift/*.ring.gz'\ --exclude='*/etc/swift/*.builder'\ --exclude='*/etc/libvirt/passwd.db'\ ${excluded_original_passwords}
+        EXCLUDE=--exclude='*/etc/swift/backups/*'\ --exclude='*/etc/swift/*.ring.gz'\ --exclude='*/etc/swift/*.builder'\ --exclude='*/etc/libvirt/passwd.db'
         # We need to repipe the tar command through 'tar xO' to force text
         # output because otherwise the sed command cannot work. The sed is
         # needed because puppet puts timestamps as comments in cron and
         # parsedfile resources, hence triggering a change at every redeploy
-        tar -c --mtime='1970-01-01' $EXCLUDE -f - /var/lib/config-data/${NAME} $additional_checksum_files | tar xO | \
+        tar -c --mtime='1970-01-01' $EXCLUDE -f - /var/lib/config-data/${NAME} | tar xO | \
                 sed '/^#.*HEADER.*/d' | md5sum | awk '{print $1}' > /var/lib/config-data/${NAME}.md5sum
-        tar -c --mtime='1970-01-01' $EXCLUDE -f - /var/lib/config-data/puppet-generated/${NAME} $additional_checksum_files --mtime='1970-01-01' | tar xO \
+        tar -c --mtime='1970-01-01' $EXCLUDE -f - /var/lib/config-data/puppet-generated/${NAME} --mtime='1970-01-01' | tar xO \
                 | sed '/^#.*HEADER.*/d' | md5sum | awk '{print $1}' > /var/lib/config-data/puppet-generated/${NAME}.md5sum
     fi
     """)
@@ -334,7 +322,7 @@ with open(sh_script, 'w') as script_file:
 
 
 def mp_puppet_config(*args):
-    (config_volume,puppet_tags,manifest,config_image,volumes,check_mode) = args[0]
+    (config_volume,puppet_tags,manifest,config_image,volumes,privileged) = args[0]
     log = get_logger()
     log.info('Starting configuration of %s using image %s' % (config_volume,
              config_image))
@@ -343,49 +331,42 @@ def mp_puppet_config(*args):
     log.debug('manifest %s' % manifest)
     log.debug('config_image %s' % config_image)
     log.debug('volumes %s' % volumes)
-    log.debug('check_mode %s' % check_mode)
-
+    log.debug('privileged %s' % privileged)
     with tempfile.NamedTemporaryFile() as tmp_man:
         with open(tmp_man.name, 'w') as man_file:
             man_file.write('include ::tripleo::packages\n')
             man_file.write(manifest)
 
-        uname = RUNNER.unique_container_name('docker-puppet-%s' %
-                                             config_volume)
-        rm_container(uname)
+        rm_container('docker-puppet-%s' % config_volume)
         pull_image(config_image)
 
-        dcmd = ['/usr/bin/docker', 'run',
+        common_dcmd = [cli_cmd, 'run',
                 '--user', 'root',
-                '--name', uname,
+                '--name', 'docker-puppet-%s' % config_volume,
                 '--env', 'PUPPET_TAGS=%s' % puppet_tags,
                 '--env', 'NAME=%s' % config_volume,
                 '--env', 'HOSTNAME=%s' % short_hostname(),
                 '--env', 'NO_ARCHIVE=%s' % os.environ.get('NO_ARCHIVE', ''),
                 '--env', 'STEP=%s' % os.environ.get('STEP', '6'),
+                '--env', 'NET_HOST=%s' % os.environ.get('NET_HOST', 'false'),
                 '--volume', '/etc/localtime:/etc/localtime:ro',
                 '--volume', '%s:/etc/config.pp:ro,z' % tmp_man.name,
                 '--volume', '/etc/puppet/:/tmp/puppet-etc/:ro,z',
-                '--volume', '/usr/share/openstack-puppet/modules/:/usr/share/openstack-puppet/modules/:ro',
-                '--volume', '%s:/var/lib/config-data/:z' % os.environ.get('CONFIG_VOLUME_PREFIX', '/var/lib/config-data'),
-                '--volume', 'tripleo_logs:/var/log/tripleo/',
-                # Syslog socket for puppet logs
-                '--volume', '/dev/log:/dev/log',
                 # OpenSSL trusted CA injection
                 '--volume', '/etc/pki/ca-trust/extracted:/etc/pki/ca-trust/extracted:ro',
                 '--volume', '/etc/pki/tls/certs/ca-bundle.crt:/etc/pki/tls/certs/ca-bundle.crt:ro',
                 '--volume', '/etc/pki/tls/certs/ca-bundle.trust.crt:/etc/pki/tls/certs/ca-bundle.trust.crt:ro',
                 '--volume', '/etc/pki/tls/cert.pem:/etc/pki/tls/cert.pem:ro',
-                # facter caching
-                '--volume', '/var/lib/container-puppet/puppetlabs/facter.conf:/etc/puppetlabs/facter/facter.conf:ro',
-                '--volume', '/var/lib/container-puppet/puppetlabs/:/opt/puppetlabs/:ro',
+                '--volume', '%s:/var/lib/config-data/:rw,z' % os.environ.get('CONFIG_VOLUME_PREFIX', '/var/lib/config-data'),
+                # Syslog socket for puppet logs
+                '--volume', '/dev/log:/dev/log:rw',
                 # script injection
-                '--volume', '%s:%s:z' % (sh_script, sh_script) ]
+                '--volume', '%s:%s:rw,z' % (sh_script, sh_script) ]
+        if privileged:
+            common_dcmd.push('--privileged')
 
-        if check_mode:
-            dcmd.extend([
-                '--volume',
-                '/etc/puppet/check-mode:/tmp/puppet-check-mode:ro'])
+
+        dcmd = common_dcmd + cli_dcmd
 
         for volume in volumes:
             if volume:
@@ -393,18 +374,18 @@ def mp_puppet_config(*args):
 
         dcmd.extend(['--entrypoint', sh_script])
 
-        env = {}
-        # NOTE(flaper87): Always copy the DOCKER_* environment variables as
-        # they contain the access data for the docker daemon.
-        for k in filter(lambda k: k.startswith('DOCKER'), os.environ.keys()):
-            env[k] = os.environ.get(k)
+        if container_cli == 'docker':
+            # NOTE(flaper87): Always copy the DOCKER_* environment variables as
+            # they contain the access data for the docker daemon.
+            for k in filter(lambda k: k.startswith('DOCKER'), os.environ.keys()):
+                env[k] = os.environ.get(k)
 
         if os.environ.get('NET_HOST', 'false') == 'true':
             log.debug('NET_HOST enabled')
             dcmd.extend(['--net', 'host', '--volume',
                          '/etc/hosts:/etc/hosts:ro'])
         dcmd.append(config_image)
-        log.debug('Running docker command: %s' % ' '.join(dcmd))
+        log.debug('Running %s command: %s' % (container_cli, ' '.join(dcmd)))
 
         subproc = subprocess.Popen(dcmd, stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE, env=env)
@@ -423,7 +404,7 @@ def mp_puppet_config(*args):
             if cmd_stderr:
                 log.debug(cmd_stderr)
             # only delete successful runs, for debugging
-            rm_container(uname)
+            rm_container('docker-puppet-%s' % config_volume)
 
         log.info('Finished processing puppet configs for %s' % (config_volume))
         return subproc.returncode
@@ -434,9 +415,6 @@ def mp_puppet_config(*args):
 # to consume.
 process_map = []
 
-check_mode = int(os.environ.get('CHECK_MODE', 0))
-log.debug('CHECK_MODE: %s' % check_mode)
-
 for config_volume in configs:
 
     service = configs[config_volume]
@@ -444,14 +422,14 @@ for config_volume in configs:
     manifest = service[2] or ''
     config_image = service[3] or ''
     volumes = service[4] if len(service) > 4 else []
+    privileged = service[5] if len(service) > 5 else False
 
     if puppet_tags:
         puppet_tags = "file,file_line,concat,augeas,cron,%s" % puppet_tags
     else:
         puppet_tags = "file,file_line,concat,augeas,cron"
 
-    process_map.append([config_volume, puppet_tags, manifest, config_image,
-                        volumes, check_mode])
+    process_map.append([config_volume, puppet_tags, manifest, config_image, volumes, privileged])
 
 for p in process_map:
     log.debug('- %s' % p)
@@ -471,16 +449,14 @@ for returncode, config_volume in zip(returncodes, config_volumes):
 
 
 # Update the startup configs with the config hash we generated above
-config_volume_prefix = os.environ.get('CONFIG_VOLUME_PREFIX', '/var/lib/config-data')
-log.debug('CONFIG_VOLUME_PREFIX: %s' % config_volume_prefix)
 startup_configs = os.environ.get('STARTUP_CONFIG_PATTERN', '/var/lib/tripleo-config/docker-container-startup-config-step_*.json')
 log.debug('STARTUP_CONFIG_PATTERN: %s' % startup_configs)
-infiles = glob.glob(startup_configs)
+infiles = glob.glob('/var/lib/tripleo-config/docker-container-startup-config-step_*.json')
 for infile in infiles:
     with open(infile) as f:
         infile_data = json.load(f)
 
-    for k, v in infile_data.iteritems():
+    for k, v in iter(infile_data.items()):
         config_volumes = match_config_volumes(config_volume_prefix, v)
         config_hashes = [get_config_hash(volume_path) for volume_path in config_volumes]
         config_hashes = filter(None, config_hashes)
@@ -493,8 +469,9 @@ for infile in infiles:
 
     outfile = os.path.join(os.path.dirname(infile), "hashed-" + os.path.basename(infile))
     with open(outfile, 'w') as out_f:
-        os.chmod(out_f.name, 0600)
+        os.chmod(out_f.name, 0o600)
         json.dump(infile_data, out_f, indent=2)
 
 if not success:
     sys.exit(1)
+
