@@ -67,7 +67,7 @@ OPTIONAL_DOCKER_SECTIONS = ['docker_puppet_tasks', 'upgrade_tasks',
                             'post_update_tasks', 'service_config_settings',
                             'host_prep_tasks', 'metadata_settings',
                             'kolla_config', 'global_config_settings',
-                            'logging_source', 'logging_groups',
+                            'logging_source',
                             'external_deploy_tasks', 'external_post_deploy_tasks',
                             'docker_config_scripts', 'step_config',
                             'monitoring_subscription',
@@ -174,10 +174,10 @@ PARAMETER_DEFINITION_EXCLUSIONS = {'CephPools': ['description',
                                                                'constraints'],
                                    # NOTE(anil): This is a temporary change and
                                    # will be removed once bug #1767070 properly
-                                   # fixed. OVN supports only VLAN, geneve
-                                   # and flat for NeutronNetworkType. But VLAN
-                                   # tenant networks have a limited support
-                                   # in OVN. Till that is fixed, we restrict
+                                   # fixed. OVN supports only VLAN and geneve
+                                   # for NeutronNetworkType. But VLAN tenant
+                                   # networks have a limited support in OVN.
+                                   # Till that is fixed, we restrict
                                    # NeutronNetworkType to 'geneve'.
                                    'NeutronNetworkType': ['description',
                                                           'default',
@@ -1075,15 +1075,13 @@ def validate(filename, param_map):
         if filename.startswith('./roles/'):
             retval = validate_role_name(filename)
 
-        if filename.startswith('./roles/ComputeHCI.yaml') or \
-                filename.startswith('./roles/ComputeHCIOvsDpdk.yaml'):
+        if filename.startswith('./roles/ComputeHCI.yaml'):
             retval |= validate_hci_computehci_role(filename, tpl)
 
         if filename.startswith('./roles/ComputeOvsDpdk.yaml') or \
                 filename.startswith('./roles/ComputeSriov.yaml') or \
                 filename.startswith('./roles/ComputeOvsDpdkRT.yaml') or \
-                filename.startswith('./roles/ComputeSriovRT.yaml') or \
-                filename.startswith('./roles/ComputeHCIOvsDpdk.yaml'):
+                filename.startswith('./roles/ComputeSriovRT.yaml'):
             exclude = [
                 'OS::TripleO::Services::OVNController',
                 'OS::TripleO::Services::ComputeNeutronOvsAgent',
@@ -1114,11 +1112,8 @@ def validate(filename, param_map):
         # NOTE(hjensas): The routed network data example is very different ...
         # We need to develop a more advanced validator, probably using a schema
         # definition instead.
-        # NOTE(mandre): Same goes for the openshift network data where it
-        # contains only a subset of the overcloud networks.
         if (filename.startswith('./network_data_') and
-                not (filename.endswith('routed.yaml') or
-                     filename.endswith('openshift.yaml'))):
+                not filename.endswith('routed.yaml')):
             result = validate_network_data_file(filename)
             retval |= result
         else:
